local Player, Enemy = table.unpack(require "adv.AdvPlayer")
local Buff = require "adv.AdvBuff"
local Passive = require "adv.AdvPassive"
local Battle = class("Battle")
function Battle:ctor(adv)
	self.adv = adv
	self.player = nil --玩家
	self.isNewPlayer = false
	self.enemys = {} --怪
	self.tempData = {} -- 临时回合数据
	self:initPlayer()
	self:initEnemys()
	self:initAfter()
	if self.isNewPlayer then
		self.player:triggerPassive(Passive.BORN_ONCE)
	end
end

function Battle:initAfter()
	self.player:initAfter(self.adv.advTeam.player)
	for _, enemy in pairs(self.enemys) do
		enemy:initAfter(self.adv.rooms[enemy.roomId].blocks[enemy.blockId].event.enemy)
	end
end
--[[
队伍总属性 = 基础属性 + 等级 × 成长率 	
	基础属性：	固定属性，与章节、队伍够成、养成无关，由策划给出	
	等级：	根据进入时的关卡，获得一个【初始等级】；消灭怪物获得经验后，累计N点会提升等级	
	羁绊加成：	加成条件与“战斗”相同，加成目标改为【冒险队】	
			
	成长率：	由【队伍总生存力】决定	
			成长率 =（总生存力 / 80）^0.52 + INT（总角色等级 / 50）* 1
--]]

function Battle:initPlayer()
	if not next(self.adv.advTeam.heros) then return end
	if not self.adv.advTeam.player then
		local player = {}
		player.passives = {}
		local heroLevel = 0
		for slot, heroId in pairs(self.adv.advTeam.heros) do
			local hero = self.adv.owner.heros[heroId]
			if hero then
				heroLevel = heroLevel + hero:getProperty("level")
				local advSkillId = csvdb["unitCsv"][self.adv.owner.heros[heroId]:getProperty("type")]["adv"]
				if advSkillId > 1000 then
					table.insert(player.passives, {id = advSkillId, level = hero:getSkillLevel(4)})
				end
			end
		end
		player.growth = (self.adv.owner:getRealBattleValue(self.adv.advTeam.heros) / 80) ^ 0.52 + math.floor(heroLevel / 50) / 50
		player.level = 1
		player.exp = 0
		player.sp = 100
		local activeRelation = self.adv.owner:getHeroActiveRelation()
		local baseAttr = csvdb["adv_unitCsv"][math.floor(self.adv.advInfo.chapter / 100)]
		for _, attr in pairs(AttsEnumEx) do
			if baseAttr[attr] then
				player[attr] = baseAttr[attr] + baseAttr[attr] * player.growth * (player.level - 1)
			end
		end
		player.hpMax = player.hp or 0
		self.adv.advTeam.player = player
		self.isNewPlayer = true
	end
	self.player = Player.new(self, self.adv.advTeam.player)
end

function Battle:initEnemys()
	for _, room in pairs(self.adv.rooms) do
		for _, block in pairs(room.blocks) do
			self:addEnemy(room, block)	
		end
	end
end

function Battle:addEnemy(room, block)
	if block.event and (block.event.etype == AdvEventType.BOSS or block.event.etype == AdvEventType.Monster) then
		if not block.event.enemy then
			local enemyCsv = csvdb["event_monsterCsv"][block.event.id]
			local enemy = {}
			enemy.hp = enemyCsv.hp + enemyCsv.levelhp * self.adv.advInfo.level
			enemy.atk = enemyCsv.atk + enemyCsv.levelatk * self.adv.advInfo.level
			enemy.hit = enemyCsv.hit + enemyCsv.levelhit * self.adv.advInfo.level
			enemy.miss = enemyCsv.miss + enemyCsv.levelmiss * self.adv.advInfo.level
			enemy.def = enemyCsv.def + enemyCsv.leveldef * self.adv.advInfo.level
			enemy.passives = {}
			for _, id in ipairs(enemyCsv.mapPassive:toArray(true, "=")) do
				table.insert(enemy.passives, {id = id})
			end
			block.event.enemy = enemy
		end
		local player = Enemy.new(self, block.event.mId or 999, block.event.id, room.roomId, block.blockId, not block.isOpen, block.event.enemy)
		table.insert(self.enemys, player)
		return player
	end 
end

function Battle:getEnemy(roomId, blockId)
	for _, enemy in ipairs(self.enemys) do
		if enemy.roomId == roomId and enemy.blockId == blockId then
			return enemy
		end
	end
end

function Battle:getEnemyById(id)
	for _, enemy in ipairs(self.enemys) do
		if enemy.id == id then
			return enemy
		end
	end
end

function Battle:getRBByEnemyId(enemyId)
	local enemy = self:getEnemyById(enemyId)
	return enemy.roomId, enemy.blockId
end


--触发全员被动技能
function Battle:triggerPassive(condType, params)
	self.player:triggerPassive(condType, params)
	for _, enemy in ipairs(self.enemys) do
		enemy:triggerPassive(condType, params)
	end
end

--回合
function Battle:afterRound()
	self.player:afterRound()
	table.sort(self.enemys, function(e1, e2)
		return e1.id < e2.id
	end)
	for _, enemy in ipairs(self.enemys) do
		enemy:afterRound()
	end
	self.player:clearRound()
	for _, enemy in ipairs(self.enemys) do
		enemy:clearRound()
	end
	for i = #self.enemys, 1, -1 do
		if self.enemys[i].isDead then
			self.adv:enemyDead(self.enemys[i].roomId, self.enemys[i].blockId)
			self.enemys[i]:clear()
			table.remove(self.enemys, i)
		end
	end

	if self.player.isDead then
		self.adv:over(false)
	end
end


function Battle:battleBegin(roomId, blockId, params)
	local enemy = self:getEnemy(roomId, blockId)
	if not enemy then return end
	local player = params.player
	-- 玩家没死就是怪死了
	if player.hp > 0 then
		enemy:hurt(enemy.hp, self.player, {hurtType = 5})
		self.player:effectBattleBuff()
	end
	self.player:hurt(math.max(0, math.ceil(self.player.hp - player.hp)), enemy, {hurtType = 5}) --战斗血量只会变少
	self.player:changeSp(math.min(0, math.floor(player.sp - self.player.sp)) , 0) --战斗魔力只会变少
end


--写入数据
function Battle:getDB()
	self.adv.advTeam.player = self.player:getDB()
	for _, enemy in ipairs(self.enemys) do
		local block = self.adv.rooms[enemy.roomId].blocks[enemy.blockId]
		block.event.enemy = enemy:getDB()
	end
end

return Battle