local skynet = require "skynet"
local harbor = require "skynet.harbor"

local table_insert = table.insert

local redisproxy = {}

setmetatable(redisproxy, { __index = function(t, k)
	local cmd = string.upper(k)
	local f = function (self, ...)
		local ok, result = pcall(skynet.call, redisd, "lua", cmd, ...)
		if not ok then
			skynet.error(cmd, ..., "\n", debug.traceback(coroutine.running(), nil))
			return
		end
		return result
	end
	t[k] = f
	return f
end})

function redisproxy:runScripts(name, ...)
	local RedisScripts = require("rdsscripts/RedisScripts")

	if not RedisScripts[name].sha1 then
		local content = io.readfile(RedisScripts[name].file)
		RedisScripts[name].sha1 = self:script("LOAD", content)
	end

	-- 不存在脚本(系统问题或者需要刷新脚本)
	local existScript = self:script("EXISTS", RedisScripts[name].sha1)
	if existScript[1] == 0 then
		local content = io.readfile(RedisScripts[name].file)
		RedisScripts[name].sha1 = self:script("LOAD", content)
	end

	return self:evalsha(RedisScripts[name].sha1, ...)
end

local meta = {__index = function (tab, name) return function (_, ...) tab[#tab+1]={name, ...} end end}
function redisproxy:pipelining(block)
	local ops = setmetatable({{"multi"}}, meta)
	block(ops)
	if #ops == 1 then return end
	ops[#ops+1]={"exec"}
	return self:pipeline(ops)
end

function redisproxy:insertEmail(params)
	local pms = {
		roleId = params.roleId,
		emailId = params.emailId,
		createtime = params.createtime or skynet.timex(),
		con1 = params.con1 or "",
		con2 = params.con2 or "",
		con3 = params.con3 or "",
		att1 = params.att1 or "",
		att2 = params.att2 or "",
		att3 = params.att3 or "",
		title = params.title or "",
		content = params.content or "",
		attachments = params.attachments or "",
	}
	self:runScripts("insertEmail", 12,
		pms.roleId, pms.emailId, pms.createtime,
		pms.con1, pms.con2, pms.con3,
		pms.att1, pms.att2, pms.att3,
		pms.title, pms.content, pms.attachments)
	return true
end

return redisproxy